package speiger.src.builder.base;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.StringJoiner;

import speiger.src.builder.mappers.IMapper;
import speiger.src.builder.misc.FileUtils;
import speiger.src.builder.misc.IdGenerator;

public class Template
{
	String fileName;
	String textFile;
	List<ConditionedSegment> segments;
	List<PostSegment> postSegments;
	
	public Template(String fileName, String textFile, List<ConditionedSegment> segments, List<PostSegment> postSegments)
	{
		this.fileName = fileName;
		this.textFile = textFile;
		this.segments = segments;
		this.postSegments = postSegments;
	}
	
	public String getFileName()
	{
		return fileName;
	}
	
	public String build(Set<String> parsePool, List<IMapper> mappers, boolean printNoWork, Set<IMapper> done)
	{
		StringBuilder builder = new StringBuilder(textFile);
		for(int i = 0,offset=0,m=segments.size();i<m;i++)
		{
			offset += segments.get(i).build(parsePool, builder, offset);
		}
		String result = builder.toString();
		for(int i = 0,m=mappers.size();i<m;i++)
		{
			if(printNoWork)
			{
				String previous = result;
				result = mappers.get(i).apply(result);
				if(previous.equals(result))
				{
					done.add(mappers.get(i));
				}
			}
			else
			{
				result = mappers.get(i).apply(result);
			}
		}
		for(int i = 0,m=postSegments.size();i<m;i++)
		{
			result = postSegments.get(i).build(result);
		}
		return result;
	}
	
	public static Template parse(Path file) throws IOException
	{
		String fileName = FileUtils.getFileName(file.getFileName());
		IdGenerator segmentIds = new IdGenerator();
		List<ConditionedSegment> segments = new ArrayList<>();
		List<PostSegment> postSegments = new ArrayList<>();
		StringJoiner joiner = new StringJoiner("\n");
		List<String> lines = Files.readAllLines(file);
		for(int i = 0;i<lines.size();i++)
		{
			String s = lines.get(i);
			String trimmed = s.trim();
			if(trimmed.startsWith("#"))
			{
				if(trimmed.startsWith("#if"))
				{
					i += ConditionedSegment.parse(fileName, s.trim().substring(3).trim(), lines, i, joiner.length(), segmentIds, segments, postSegments);
					continue;
				}
				else if(trimmed.startsWith("#ignore"))
				{
					segmentIds.appendId();
					int prev = i;
					i += ConditionedSegment.parse(fileName, null, lines, i, joiner.length(), segmentIds, segments, postSegments);
					joiner.add(lines.get(prev));
					continue;					
				}
				else if(trimmed.startsWith("#symlink"))
				{
					return Template.parse(file.getParent().resolve(trimmed.substring(8).trim()));
				}
			}
			joiner.add(s);
		}
		return new Template(fileName, joiner.toString(), segments, postSegments);
	}
}
